Class: CloudCrowd::NodeRecord

Inherits:
ActiveRecord::Base
  • Object
show all
Defined in:
lib/cloud_crowd/models/node_record.rb

Overview

A NodeRecord is the central server's record of a Node running remotely. We can use it to assign WorkUnits to the Node, and keep track of its status. When a Node exits, it destroys this record.

Defined Under Namespace

Classes: Serializer

Constant Summary collapse

PORT =

Extract the port number from the host id.

/:(\d+)\Z/

Class Method Summary collapse

Instance Method Summary collapse

Class Method Details

.check_in(params, request) ⇒ Object

Register a Node with the central server. This happens periodically (once every Node::CHECK_IN_INTERVAL seconds). Nodes will be de-registered if they checked in within a reasonable interval.



26
27
28
29
30
31
32
33
34
35
36
# File 'lib/cloud_crowd/models/node_record.rb', line 26

def self.check_in(params, request)
  attrs = {
    :ip_address       => request.ip,
    :port             => params[:host].match(PORT)[1].to_i,
    :busy             => params[:busy],
    :tag              => params[:tag],
    :max_workers      => params[:max_workers],
    :enabled_actions  => params[:enabled_actions]
  }
  self.find_or_create_by(:host => params[:host]).update_attributes!(attrs)
end

Instance Method Details

#actions ⇒ Object

What Actions is this Node able to run?



56
57
58
# File 'lib/cloud_crowd/models/node_record.rb', line 56

def actions
  @actions ||= enabled_actions.split(',')
end

#active_model_serializer ⇒ Object



107
# File 'lib/cloud_crowd/models/node_record.rb', line 107

def active_model_serializer; Serializer; end

#busy? ⇒ Boolean

Is this Node too busy for more work? Determined by number of workers, or the Node's load average, as configured in config.yml.

Returns:

  • (Boolean)


62
63
64
# File 'lib/cloud_crowd/models/node_record.rb', line 62

def busy?
  busy || (max_workers && work_units.count >= max_workers)
end

#display_status ⇒ Object

The printable status of the Node.



79
80
81
# File 'lib/cloud_crowd/models/node_record.rb', line 79

def display_status
  busy? ? 'busy' : 'available'
end

#node ⇒ Object

Keep a RestClient::Resource handy for contacting the Node, including HTTP authentication, if configured.



74
75
76
# File 'lib/cloud_crowd/models/node_record.rb', line 74

def node
  @node ||= RestClient::Resource.new(url, CloudCrowd.client_options)
end

#release_work_units ⇒ Object

Release all of this Node's WorkUnits for other nodes to take.



89
90
91
# File 'lib/cloud_crowd/models/node_record.rb', line 89

def release_work_units
  WorkUnit.where("node_record_id = #{id}").update_all('node_record_id = null, worker_pid = null')
end

#send_work_unit(unit) ⇒ Object

Dispatch a WorkUnit to this node. Places the node at back at the end of the rotation. If we fail to send the WorkUnit, we consider the node to be down, and remove this record, freeing up all of its checked-out work units. If the Node responds that it's overloaded, we mark it as busy. Returns true if the WorkUnit was dispatched successfully.



43
44
45
46
47
48
49
50
51
52
53
# File 'lib/cloud_crowd/models/node_record.rb', line 43

def send_work_unit(unit)
  result = node['/work'].post(:work_unit => unit.to_json)
  unit.assign_to(self, JSON.parse(result.body)['pid'])
  touch && true
rescue RestClient::RequestFailed => e
  raise e unless e.http_code == 503 && e.http_body == Node::OVERLOADED_MESSAGE
  update_attribute(:busy, true) && false
rescue RestClient::Exception, Errno::ECONNREFUSED, Timeout::Error, RestClient::RequestTimeout, Errno::ECONNRESET
  # Couldn't post to node, assume it's gone away.
  destroy && false
end

#to_json ⇒ Object



109
110
111
# File 'lib/cloud_crowd/models/node_record.rb', line 109

def to_json
  Serializer.new(self).to_json
end

#url ⇒ Object

The URL at which this Node may be reached. TODO: Make sure that the host actually has externally accessible DNS.



68
69
70
# File 'lib/cloud_crowd/models/node_record.rb', line 68

def url
  @url ||= "http://#{host}"
end

#worker_pids ⇒ Object

A list of the process ids of the workers currently being run by the Node.



84
85
86
# File 'lib/cloud_crowd/models/node_record.rb', line 84

def worker_pids
  work_units.pluck('worker_pid')
end